// Node editor UI element

use glium::glutin::window::Window as GlutinWindow;
use glium::glutin::dpi::LogicalPosition;
use imgui::{Ui, MouseButton};
use imgui::color::ImColor32;
use imgui::draw_list::DrawListMut;

const COLOR_BLUE: ImColor32 = ImColor32::from_rgb(76u8, 177u8, 230u8);

pub struct DragArea {
    pub offset: [f32; 2],
    pub size: [f32; 2],
    pub border_color: ImColor32,
    default_offset: [f32; 2],
    default_size: [f32; 2]
}

pub fn init(
    offset: [f32; 2],
    size: [f32; 2]
) -> DragArea {
    DragArea {
        offset,
        size,
        border_color: COLOR_BLUE,
        default_offset: offset,
        default_size: size
    }
}

impl DragArea {
    pub fn build
        <F: FnMut(&DrawListMut, &[f32; 2], &[f32; 2], &[f32; 2]) + 'static>
        (&mut self, ui: &Ui, window: &GlutinWindow, mut build_children: F)
    {
        let mouse_delta = ui.io().mouse_delta;
        let mouse_pos = ui.io().mouse_pos;
        
        if ui.button("reset view") {
            self.offset = self.default_offset;
        }

        ui.invisible_button("##empty", self.size);

        let rect_min = ui.item_rect_min();
        let rect_max = ui.item_rect_max();

        let lmb_down = ui.is_mouse_dragging(MouseButton::Left);
        let mmb_down = ui.is_mouse_dragging(MouseButton::Middle);

        if ui.is_item_active() && (lmb_down || mmb_down) {
            if rect_min[0] > mouse_pos[0] {
                window.set_cursor_position(
                    LogicalPosition { x: rect_max[0], y: mouse_pos[1] }
                ).unwrap();
                self.offset[0] -= self.size[0];
            }

            if rect_max[0] < mouse_pos[0] {
                window.set_cursor_position(
                    LogicalPosition { x: rect_min[0], y: mouse_pos[1] }
                ).unwrap();
                self.offset[0] += self.size[0];
            }

            if rect_min[1] > mouse_pos[1] {
                window.set_cursor_position(
                    LogicalPosition { x: mouse_pos[0], y: rect_max[1] }
                ).unwrap();
                self.offset[1] -= self.size[1];
            }

            if rect_max[1] < mouse_pos[1] {
                window.set_cursor_position(
                    LogicalPosition { x: mouse_pos[0], y: rect_min[1] }
                ).unwrap();
                self.offset[1] += self.size[1];
            }
            
            self.offset[0] += mouse_delta[0];
            self.offset[1] += mouse_delta[1];
        }

        let draw_list = ui.get_window_draw_list();

        draw_list.with_clip_rect(rect_min, rect_max, || {
            build_children(&draw_list, &rect_min, &rect_max, &self.offset);
            draw_list.add_rect(rect_min, rect_max, self.border_color).build();
        });
    }
}
