use glium::glutin::dpi::LogicalSize;
use imgui::*;
use imgui::color::ImColor32;

mod app;
mod drag_area;

const INIT_SIZE: (f64, f64) = (640.0, 480.0);
const MIN_SIZE: (f64, f64) = (300.0, 250.0);


fn main() {
    let main_app = app::init("muh window", INIT_SIZE);

    main_app.display
        .gl_window()
        .window()
        .set_min_inner_size(Some(LogicalSize::new(MIN_SIZE.0, MIN_SIZE.1)));

    let mut main_drag_area = drag_area::init(
        [30.0, 30.0],
        [200.0, 200.0]
    );

    main_app.main_loop(move |_, ui, display| {
        Window::new("Node Editor")
            .size([300.0, 110.0], Condition::FirstUseEver)
            .build(&ui, || {
                ui.text("Basic Drag Area");
                ui.separator();

                main_drag_area.build(
                    &ui,
                    &display.gl_window().window(),
                    move |draw_list, area_min, area_max, offset| {
                        let text_pos: [f32; 2] = [
                            area_min[0] + offset[0],
                            area_min[1] + offset[1]
                        ];

                        draw_list.add_text(
                            text_pos,
                            ImColor32::WHITE,
                            "Basic Drag Area Contents"
                        );
                    }
                );

                draw_mouse_pos_text(ui);
            });
    });
}

fn draw_mouse_pos_text(ui: &Ui) {
    let mouse_pos = ui.io().mouse_pos;

    ui.text(format!(
        "mouse pos: ({:.1}, {:.1})",
        mouse_pos[0], mouse_pos[1]
    ));
}
